/*
 * CS_Color_namer.c
 * Software to accompany the article -
 * "The Quark D2000 Development Board: I2C Interfacing"
 * Demonstration of the Flora TSC34725 color sensor
 * with a Newhaven C022BiZ-FSW-FBW-3VM 2 X 20 COG display
 *********************************************************
 ***  This software is offered strictly as is with no  ***
 ***  warranties whatsoever. Use it at your own risk.  ***
 *********************************************************
 */

#include "bsp/drivers/include/clk.h"
#include "bsp/drivers/include/qm_gpio.h"
#include "bsp/drivers/include/qm_i2c.h"
#include "bsp/drivers/include/qm_pinmux.h"
#include <string.h>

#include "C0220BiZ_Lib.h"

/* defines */
#define CSaddr (0x29)	/* Color sensor I2C address */

/* function prototypes */
int cs_init(void);
int cs_read(void);

/* Globals */
static qm_i2c_status_t status;		/* see i2c.h for values 						*/
uint16_t redc,greenc,bluec,clearc;	/* ADC channel values 							*/
uint8_t sensor[10];                 /* hold the sensor read data c/r/g/b/lsb/msb	*/
uint8_t cs_pu[2]={0x00,0x01};		/* power up the TCS chip          				*/
uint8_t cs_en[2]={0x00,0x03};       /* power on and ADC on            				*/
uint8_t cs_it[2]={0x01,0xd5};       /* set integration time to 101 ms 				*/
									/* note: leave gain at power up default 		*/
uint8_t cs_cc[1]={0xb4};            /* clear channel set for block read    			*/

/* GPIO configuration structure (see qm_gpio.h for details) */
static qm_gpio_port_config_t portcfg;
/*  I2C structure variables  */
static qm_i2c_config_t I2Ccfg;

int main() {

	float x,y,z;
	float xcc,ycc;
	int color=0;
	/* color look up table xy
	 * 1=Red
	 * 2=Orange
	 * 3=Yellow
	 * 4=Green
	 * 5=Blue
	 * 6=not used
	 * 7=Violet
	 * 8=Black/White
	 */
	int colors[8][9]=
	        {
			{5,5,5,5,4,4,4,4,4},
			{5,5,5,5,4,4,4,4,4},
			{7,5,5,5,4,4,4,4,4},
			{7,7,1,8,4,4,4,4,4},
			{1,1,1,3,3,3,4,4,4},
			{1,1,1,1,2,2,1,1,1},
			{1,1,1,1,1,1,1,1,1},
			{1,1,1,1,1,1,1,1,1}
	        };

	/* configure GPIO RST bit to output by setting the associated bit in the DDR */
	portcfg.direction = 1 << 5; /* equivalent to portcfg.direction = BIT(LED);*/
	qm_gpio_set_config(QM_GPIO_0, &portcfg);
	qm_gpio_set_pin(QM_GPIO_0, LCDRST); /* set LCDRST high (reset is active low) */

	/* set up for I2C IRQ
	qm_irq_request(QM_IRQ_I2C_0, qm_i2c_0_isr);
	*/
	/*  enable I2C 0 */
	clk_periph_enable(CLK_PERIPH_CLK | CLK_PERIPH_I2C_M0_REGISTER);

	/* set IO pins for SDA and SCL */
	qm_pmux_select(QM_PIN_ID_6, QM_PMUX_FN_2);
	qm_pmux_select(QM_PIN_ID_7, QM_PMUX_FN_2);

	/* configure I2C */
	I2Ccfg.address_mode = QM_I2C_7_BIT;
	I2Ccfg.mode = QM_I2C_MASTER;
	I2Ccfg.speed = QM_I2C_SPEED_STD;
	if (qm_i2c_set_config(QM_I2C_0, &I2Ccfg)) {
		return 1; /* exit */
	}
	LCD_reset();
	LCD_init();
	LCD_clr();
    LCD_writexy(2,0,"Color Identifier");
	clk_sys_udelay(3000000);
	if(cs_init())
		{
		LCD_writexy(0,1,"ERROR in cs_init()");
		clk_sys_udelay(5000000);
		}
	while(1)
	{
		if(cs_read())
			{
			LCD_writexy(0,1,"ERROR in cs_read()");
			clk_sys_udelay(5000000);
			}
		/* convert the sensor values */
		/* see the TAO tech note for where these values came from */
		x = ( -0.14282 * redc ) + ( 1.54924 * greenc ) + ( -0.95641 * bluec );
		y = ( -0.32466 * redc ) + ( 1.57837 * greenc ) + ( -0.73191 * bluec );
		z = ( -0.68202 * redc ) + ( 0.77073 * greenc ) + (  0.56332 * bluec );
		xcc = x/(x+y+z);
		ycc = y/(x+y+z);
		/* fake some rounding */
		xcc+=.05;
		ycc+=.05;

		/* trim excess if it occurs */
		if (xcc>0.7)
		{
			xcc=0.7;
		}
		if (ycc>0.8)
		{
			ycc=0.8;
		}
		xcc*=10.0;
		ycc*=10.0;
		/* look up the color number for the map */
		color=colors[( (int)(xcc) )][( (int)(ycc) ) ];
		switch (color) {
			case 1:
				LCD_writexy(0,1,"         RED        ");
				break;
			case 2:
				LCD_writexy(0,1,"       ORANGE       ");
				break;
			case 3:
				LCD_writexy(0,1,"       YELLOW       ");
				break;
			case 4:
				LCD_writexy(0,1,"        GREEN       ");
				break;
			case 5:
				LCD_writexy(0,1,"        BLUE        ");
				break;
			case 7:
				LCD_writexy(0,1,"       VIOLET       ");
				break;
			case 8:
				LCD_writexy(0,1,"     White/Black    ");
				break;
			default:
				LCD_writexy(0,1,"   ** ????????? **  ");
				break;
		}
		/* uncomment if you want to see the values display on the serial monitor */
		/*
		QM_PRINTF("xcc=%d ycc=%d color=%d\n",((int)xcc),((int)ycc),color);
		*/
		clk_sys_udelay(500000);
	}

	return 0;
}

int cs_init()
{
	/* will return 0 upon success or errno on first failure*/

	/* set integration time */
	if (qm_i2c_master_write(QM_I2C_0, CSaddr, cs_it, 2, true, &status)) {
		return (errno);
	}
	/* set PON */
	if (qm_i2c_master_write(QM_I2C_0, CSaddr, cs_pu, 2, true, &status)) {
		return (errno);
	}
	/* delay for 5 ms for PON to come up */
	clk_sys_udelay(5000);

	/* PON plus enable ADCs to start  */
	if (qm_i2c_master_write(QM_I2C_0, CSaddr, cs_en, 2, true, &status)) {
		return (errno);
	}
	/* delay for a conversion */
	clk_sys_udelay(110000);
	return (0);
}
int cs_read()
{
	/* will return 0 upon success or errno */

    /* set address to clear channel */
	if (qm_i2c_master_write(QM_I2C_0, CSaddr, cs_cc, 1, true, &status)) {
		return (errno);
	}
	/* read 8 bytes of sensor data into sensor[] */
	if (qm_i2c_master_read(QM_I2C_0, CSaddr, sensor, 8, true, &status)) {
		return (errno);
	}
	/* get sensor data into clearc/redc/greenc/bluec lsb-msb */
	clearc=sensor[1];
	clearc=clearc<<8;
	clearc|=sensor[0];
	redc=sensor[3];
	redc=redc<<8;
	redc|=sensor[2];
	greenc=sensor[5];
	greenc=greenc<<8;
	greenc|=sensor[4];
	bluec=sensor[7];
	bluec=bluec<<8;
	bluec|=sensor[6];

	return(0);
}




